/* Copyright (c) 2018 Albert S.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#include "handlerallpages.h"



Response HandlerAllPages::handle(const Request &r)
{
	try
	{
		Response response;
		auto pageDao = this->database->createPageDao();
		QueryOption qo = queryOption(r);
		auto resultList = pageDao->getPageList(qo);
		if(resultList.size() == 0)
		{
			return errorResponse("No pages", "This wiki does not have any pages yet");
		}
		TemplatePage &searchPage = this->templ->getPage("allpages");
		std::string body = this->templ->renderSearch(resultList);
		searchPage.setVar("pagelist", body);
		setGeneralVars(searchPage);
		response.setBody(searchPage.render());
		response.setStatus(200);
		return response;
	}
	catch(std::exception &e)
	{
		Logger::error() << "Error during allpages Handler" << e.what();
		return errorResponse("Error", "An unknown error occured");
	}

}
